/*
 * Created on Jun 11, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import java.util.Iterator;
import java.util.Map;

/**
 * Static-method implementation of the Perl <code>chop</code> keyword.
 * 
 * <code>chop</code> removes the last character off of a string (or in this
 * case, a <code>StringBuffer</code>). The return value is the character that
 * was removed. If an array of <code>StringBuffer</code> objects are given as
 * the argument, all are operated on but only the character from the very last
 * one is returned.
 * <p>
 * Here is a simple example, based on creating one string from an array of
 * strings with a given character separator (a task better and more efficiently
 * done by {@link Join}, but useful as an example):
 * <pre>
 *     import com.blackperl.Perl.Chop;
 * 
 *     StringBuffer buffer = new StringBuffer();
 *     for (int index; index < stringlist.length; index++)
 *     {
 *         buffer.append(stringlist[index]).append(',');
 *     }
 *     // Use chop to remove the extra comma at the end:
 *     Chop.chop(buffer);
 * </pre>
 * <p>
 * To keep with the Perl idiom this emulates, the argument may also be an
 * object that implements the <code>Map</code> interface. If this is the case,
 * all the <i>values</i> (not keys) must be <code>StringBuffer</code> objects.
 * All will be chopped, just as with the array-form above (which also means
 * that the return value is the character from the last chop). This is the
 * only form of <code>chop</code> that might throw a significant exception, in
 * the case where the value in a <code>Map</code> entry cannot be cast to
 * <code>StringBuffer</code>.
 * <p>
 * All components in this package provide an <code>instance</code> method to
 * retrieve a singleton object which may be used to call the static methods,
 * if the programmer prefers using an object to static invocation.
 * <p>
 * If this is the JDK 1.5 ("Tiger") edition of the package, this class is
 * suitable for use via static import:
 * <pre>
 *     import com.blackperl.Perl.Chop.chop;
 * 
 *     StringBuffer buffer = new StringBuffer();
 *     for (int index; index < stringlist.length; index++)
 *     {
 *         buffer.append(stringlist[index]).append(',');
 *     }
 *     // Use chop to remove the extra comma at the end:
 *     chop(buffer);
 * </pre>
 * 
 * @author rjray
 * @version $Id: Chop.java,v 1.7 2004/07/20 08:31:17 rjray Exp $
 */
public final class Chop
{
    private static Chop singleton;
    
    /**
     * Basic constructor, explicitly declared here as <code>private</code> to
     * prevent other classes instantiating this class.
     */
    private Chop()
    {
        super();
    }
    
    /**
     * The <code>instance</code> method is used to retrieve the
     * <code>Chop</code> singleton that applications can use in lieu of
     * invoking the methods statically.
     * 
     * @return  Singleton instance of the Chomp class
     */
    public synchronized static Chop instance()
    {
        if (singleton == null)
        {
            singleton = new Chop();
        }
        
        return(singleton);
    }
    
    /**
     * Chop the trailing character off of the <code>StringBuffer</code>
     * object passed in.
     * 
     * @param buffer    StringBuffer to trim from
     * @return  The character removed (or '\0' if buffer is empty) 
     */
    public static char chop(StringBuffer buffer)
    {
        char retval = '\0';
        
        if (buffer.length() > 0)
        {
            retval = buffer.charAt(buffer.length() - 1);
            buffer.deleteCharAt(buffer.length() - 1);
        }
        
        return retval;
    }
    
    /**
     * Chop all the <code>StringBuffer</code> objects in the array.
     * 
     * @param buffers   Array of StringBuffer to operate on
     * @return  The character from the last chop()
     */
    public static char chop(StringBuffer[] buffers)
    {
        char retval = '\0';
        
        for (int i = 0; i < buffers.length; i++)
        {
            retval = chop(buffers[i]);
        }

        return retval;
    }
    
    /**
     * Chop all the <code>StringBuffer</code> objects that are the values of
     * the <code>Map</code> object passed in. Throws an exception if any of
     * them cannot be cast to <code>StringBuffer</code>.
     * 
     * @param table An object implementing the Map interface
     * @return  The character from the last chop().
     * @throws ClassCastException
     */
    public static char chop(Map table) throws ClassCastException
    {
        char retval = '\0';
        StringBuffer obj;
        
        for (Iterator iter = table.values().iterator(); iter.hasNext(); )
        {
            obj = (StringBuffer)iter.next();
            retval = chop(obj);
        }
        
        return retval;
    }
}
