/*
 * Created on Jun 11, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

/**
 * Interface expected by methods in the <code>Grep</code> class.
 * 
 * When using the <code>grep</code> function (see {@link Grep}), the data
 * returned by the evaluation function has a flexible relationship to the
 * data provided as input. While many uses of <code>grep</code> simply prune
 * a list using boolean expressions, it is also possible to transform the
 * data as well. Because of this, the interface method returns an object
 * rather than a boolean value.
 * <p>
 * The <code>grep</code> function operates in one of two modes: testing and
 * collating. When <code>grep</code> is collating results, those inputs for
 * which the output is not <code>null</code> are added to the resulting list.
 * When <code>grep</code> is "testing", each return value from <code>eval</code>
 * must be an instance (or derivative) of <code>Boolean</code>. Those input
 * elements for which the return value is "true" are added to the resulting
 * list. This has the disadvantage that <code>eval</code> cannot return objects
 * of type <code>Boolean</code> in collating mode.
 * <p>
 * One of the many ways in which Java and Perl differ functionally is that Java
 * does not have support for the notion of "closures", or anonymous
 * lexically-scoped blocks. Some aspects of this can be emulated using
 * private or anonymous inner-classes. However, this is not a complete
 * emulation, since these blocks can only access values from their enclosing
 * scope that are marked <b>static</b> and <b>final</b>. This makes it a
 * challenge to use this interface (or {@link MapBlock}) in the fashion most
 * Perl developers are accustomed to.
 * <p>
 * The answer lies in extending the interface, or implementing it in an
 * inner-class. Take for example the <code>RegexpBlock</code> inner-class from
 * the {@link Grep} class:
 * <pre>
 *     private final class RegexpBlock implements GrepBlock
 *     {
 *         private Pattern pat;
 *         
 *         protected RegexpBlock(final Pattern p)
 *         {
 *             pat = p;
 *         }
 *             
 *         public Object eval(final Object o)
 *         {
 *             return Boolean.valueOf(pat.matcher(o.toString()).matches());
 *         }
 *     }
 * </pre>
 * This example implements only a single constructor which itself handles the
 * initialization of the <code>java.util.regex.Pattern</code> instance variable
 * <code>pat</code>. Then the <code>eval</code> method is defined as being in
 * the <i>testing</i> mode (as defined earlier), in that it returns objects that
 * are of the <code>Boolean</code> class. This implementation gets around the
 * fact that an anonymous instance of <code>GrepBlock</code> created as part
 * of a call to <code>grep</code> would not be able to refer to a specific
 * pattern, unless it were visible to the current lexical scope as a <b>final
 * static</b> value. As a fully-defined class, it is simple to add more member
 * values and even accessors or other methods entirely. While this is still not
 * the same as having true lexical closures, it will at least get the job done.
 * <p>
 * This interface may also be used in conjunction with the <code>map</code>
 * function (see {@link Map}).
 * 
 * @author rjray
 * @version $Id: GrepBlock.java,v 1.8 2004/07/20 08:31:17 rjray Exp $
 */
public interface GrepBlock
{
    /**
     * When the <code>grep</code> (or <code>map</code>) function is called with
     * an object implementing this interface, this is the function it will
     * expect to be able to invoke for each element in the list it also
     * receives. The implementing class should provide this such that it takes
     * a single argument, and returns a single object (or null, which is also
     * acceptable).
     * 
     * @param object    An arbitrary object passed in by <code>grep</code> (or
     * <code>map</code>)
     * @return  An arbitrary object, based on the functionality this
     * method implements
     */
    Object eval(Object object); 
}
